/*
    $Id: kticker.cpp,v 1.1 1998/08/05 02:10:05 paul Exp paul $

    kTicker, a news headlines display program.
    
    Copyright (C) 1998 Paul Dwerryhouse   paul@xenu.ee.unimelb.edu.au

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// Define this if links don't come up properly when you click on them.
// Thanks to Chi-hsuan Lai (lai@physics.utexas.edu) for this patch.

#undef STRANGE_DESKTOP_PROBLEM

#include <kfm.h>
#include <kconfig.h>
#include <kapp.h>
#include <kcontainer.h>  
#include <kbutton.h>  


#include <qlabel.h>
#include <qpushbt.h>
#include <qtimer.h>
#include <qpopmenu.h>
#include <qpalette.h> 
#include <qdatetm.h> 
#include "kticker.h"
#include "kticker.moc"
#include "NewsSevenAM.h"
#include "NewsBBC.h"
#include "NewsSlashDot.h"
#include "ItemList.h"
#include "TickerItem.h"
#include "KtOptBox.h"
#include "TickWidget.h"

extern "C" {
#include <stdio.h>
#include <unistd.h>
} 

KApplication *mykapp;
item *getheadlines(char *);
char *getline(FILE *);
KConfig *myconfig;
QString newsfeed;
QTime currtime;
int changetime;
KButton *button;

int main(int argc, char **argv)
{
    int retval;

    mykapp = new KApplication(argc, argv, "kticker");
    myconfig = mykapp->getConfig();
    newsfeed = myconfig->readEntry("newsfeed","bbc");
    changetime = myconfig->readNumEntry("changetime",3000);
    if (changetime<3000) {
    	changetime=3000;
	myconfig->writeEntry("changetime",changetime);
    }

    KTicker *s = new KTicker();

    mykapp->setMainWidget(s);
    s->show();
    retval = mykapp->exec();

    return retval;
}

KTicker::KTicker() : QWidget () {
    	kfm=0;
	havedata=0;
	headlines=0; current=0;
	il=0;
	currentItem=0;

	QColorGroup qcg ( QColor(255,255,255), 
		QColor(0,0,0), QColor(0,0,0), 
		QColor(0,0,0), QColor(0,0,0),
		QColor(255,255,255), QColor(0,0,0));

	QPalette qp (qcg,qcg,qcg);

	popmenu = new QPopupMenu(0,"Pop");
	popmenu->insertItem("Open");
	popmenu->insertItem("BBC");
	popmenu->insertItem("7am.com");
	popmenu->insertItem("SlashDot.Org");
	popmenu->insertItem("Options");
	popmenu->insertItem("Save Options");
	popmenu->insertItem("Quit");
	
	kttest = new KtOptBox(myconfig);
	connect(kttest,SIGNAL(changedopts(KConfig *)),this
		,SLOT(slotChangeOpts(KConfig *)));

	popmenu->connectItem(0,this,SLOT(slotOpenLink()));
	popmenu->connectItem(1,this,SLOT(restartBBC()));
	popmenu->connectItem(2,this,SLOT(restartSevenAM()));
	popmenu->connectItem(3,this,SLOT(restartSlashDot()));
	popmenu->connectItem(4,this,SLOT(configOptions()));
	popmenu->connectItem(5,this,SLOT(saveOptions()));
	popmenu->connectItem(6,this,SLOT(quitApp()));

	setMaximumSize(420,40);
	tw = new TickWidget(this);
	tw->setText("Loading data...");
	connect(tw,SIGNAL(clicked(QMouseEvent *)),this
		,SLOT(slotClickLink(QMouseEvent *)));
	connect(tw,SIGNAL(stopTicker()),this
		,SLOT(slotStopTicker()));
	connect(tw,SIGNAL(startTicker()),this
		,SLOT(slotStartTicker()));
	tw->show();

	currtime.start();
	
	if (newsfeed == "7am") {
		news = new NewsSevenAM();
		((NewsSevenAM *) news)->getNews();
	} else if (newsfeed == "slashdot") {
		news = new NewsSlashDot();
		((NewsSlashDot *) news)->getNews();
	} else {
		news = new NewsBBC();
		((NewsBBC *) news)->getNews();
	}

	timer = new QTimer(this);
	connect(timer,SIGNAL(timeout()),this,SLOT(slotTimeSignal()));
	timer->start(changetime,0);
}

void KTicker::setText(char *s) {
	tw->setText(s);
}

void KTicker::slotTimeSignal() {
	
	if (!news) return;

	// If data hasn't been loaded, then just exit.
	if (!news->isReady()) return;

	// If we haven't read the data yet, read it.
	if (!il) il=news->getItemList();

	if (!il->getNum()) {
		tw->setText("No connection");
		//setText("No connection");
		return;
	}

	// Get the next item.
	currentItem=il->next();

	// Check to see if we're at the end of the list of items.
	// If yes, then load up new data.
	if (!currentItem) {
		il = news->getItemList();;	

		// Rob Malda of SlashDot fame requests that tickers
		// suck down the SlashDot news no more often than once
		// every thirty minutes. I've made it once every hour,
		// to be on the safe side. Be nice and leave it like
		// this -- otherwise he might take it away from us.
		// (Slashdot never seems to get updated more than once
		// an hour anyway).
		if (newsfeed != "slashdot" || (currtime.elapsed())>3600000) {
			il = 0;	
			news->getNews();
			tw->setText("Loading new data...");
			//setText("Loading new data...");
			currtime.restart();
		}
		return;
	}

	// Write the current item's headline onto the display
	tw->setText(currentItem->getHeadline());
	//setText(currentItem->getHeadline());
}

// This routine is called when the kticker is clicked on.
void KTicker::slotClickLink(QMouseEvent *me) {

#ifndef STRANGE_DESKTOP_PROBLEM
	KFM *webkfm;
#endif

	// If the right button was used, pop up a menu
	if (me && me->button()==RightButton) {
		popmenu->popup( mapToGlobal( me->pos() ) );
	}

	// Otherwise go get the relevent web page
	else if (currentItem && currentItem->getUrl()) {
#ifndef STRANGE_DESKTOP_PROBLEM
    		webkfm = new KFM;
    		webkfm->openURL(currentItem->getUrl());
		delete webkfm;
#else 
		char cmd[160];
		sprintf(cmd, "kfmclient folder %s", currentItem->getUrl() );
		system(cmd);
#endif
	}
}

void KTicker::slotOpenLink() {
	slotClickLink(0);
}

void KTicker::slotStopTicker() {
	timer->stop();
	tw->setStopped();
}

void KTicker::slotStartTicker() {
	timer->start(changetime,0);
	tw->setPlay();
}

void KTicker::configOptions() {
	kttest->show();
}

void KTicker::slotChangeOpts(KConfig *cfg) {
    	changetime = cfg->readNumEntry("changetime",3000);
	timer->start(changetime,0);
}

void KTicker::saveOptions() {
	myconfig->sync();
}

void KTicker::quitApp() {
	exit(0);
}

void KTicker::restartBBC() {
	timer->stop();
	newsfeed="bbc";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsBBC();
	((NewsBBC *) news)->getNews();
	timer->start(changetime,0);
}

void KTicker::restartSevenAM() {
	timer->stop();
	newsfeed="7am";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsSevenAM();
	((NewsSevenAM *) news)->getNews();
	timer->start(3000,0);
}

void KTicker::restartSlashDot() {
	timer->stop();
	newsfeed="slashdot";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsSlashDot();
	((NewsSlashDot *) news)->getNews();
	timer->start(changetime,0);
}

void KTicker::enterEvent(QEvent *) {

	if (currentItem && currentItem->getUrl()) {
		tw->setHiColour();
	}
}

void KTicker::leaveEvent(QEvent *) {

	tw->setLoColour();
}
